/*-
 * Copyright (c) 2007 Robert N. M. Watson
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $FreeBSD: release/9.1.0/usr.bin/procstat/procstat_args.c 230917 2012-02-02 18:17:49Z trociny $
 */

#include <sys/param.h>
#include <sys/sysctl.h>
#include <sys/user.h>

#include <err.h>
#include <errno.h>
#include <libprocstat.h>
#include <limits.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "procstat.h"

static char args[ARG_MAX];

static void
do_args(struct kinfo_proc *kipp, int env)
{
	int error, name[4];
	size_t len;
	char *cp;

	if (!hflag)
		printf("%5s %-16s %-53s\n", "PID", "COMM",
		    env ? "ENVIRONMENT" : "ARGS");

	name[0] = CTL_KERN;
	name[1] = KERN_PROC;
	name[2] = env ? KERN_PROC_ENV : KERN_PROC_ARGS;
	name[3] = kipp->ki_pid;
	len = sizeof(args);
	error = sysctl(name, 4, args, &len, NULL, 0);
	if (error < 0 && errno != ESRCH && errno != EPERM) {
		warn("sysctl: kern.proc.%s: %d: %d", env ? "env" : "args",
		    kipp->ki_pid, errno);
		return;
	}
	if (error < 0)
		return;
	if (len == 0 || strlen(args) == 0) {
		strcpy(args, "-");
		len = strlen(args) + 1;
	}

	printf("%5d ", kipp->ki_pid);
	printf("%-16s ", kipp->ki_comm);
	for (cp = args; cp < args + len; cp += strlen(cp) + 1)
		printf("%s%s", cp != args ? " " : "", cp);
	printf("\n");
}

void
procstat_args(struct kinfo_proc *kipp)
{
	do_args(kipp, 0);
}

void
procstat_env(struct kinfo_proc *kipp)
{
	do_args(kipp, 1);
}
