
/*
 * debug.c
 *
 * Copyright (c) 1996-1999 Whistle Communications, Inc.
 * All rights reserved.
 * 
 * Subject to the following obligations and disclaimer of warranty, use and
 * redistribution of this software, in source or object code forms, with or
 * without modifications are expressly permitted by Whistle Communications;
 * provided, however, that:
 * 1. Any and all reproductions of the source or object code must include the
 *    copyright notice above and the following disclaimer of warranties; and
 * 2. No rights are granted, in any manner or form, to use Whistle
 *    Communications, Inc. trademarks, including the mark "WHISTLE
 *    COMMUNICATIONS" on advertising, endorsements, or otherwise except as
 *    such appears in the above copyright notice or in the software.
 * 
 * THIS SOFTWARE IS BEING PROVIDED BY WHISTLE COMMUNICATIONS "AS IS", AND
 * TO THE MAXIMUM EXTENT PERMITTED BY LAW, WHISTLE COMMUNICATIONS MAKES NO
 * REPRESENTATIONS OR WARRANTIES, EXPRESS OR IMPLIED, REGARDING THIS SOFTWARE,
 * INCLUDING WITHOUT LIMITATION, ANY AND ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.
 * WHISTLE COMMUNICATIONS DOES NOT WARRANT, GUARANTEE, OR MAKE ANY
 * REPRESENTATIONS REGARDING THE USE OF, OR THE RESULTS OF THE USE OF THIS
 * SOFTWARE IN TERMS OF ITS CORRECTNESS, ACCURACY, RELIABILITY OR OTHERWISE.
 * IN NO EVENT SHALL WHISTLE COMMUNICATIONS BE LIABLE FOR ANY DAMAGES
 * RESULTING FROM OR ARISING OUT OF ANY USE OF THIS SOFTWARE, INCLUDING
 * WITHOUT LIMITATION, ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * PUNITIVE, OR CONSEQUENTIAL DAMAGES, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES, LOSS OF USE, DATA OR PROFITS, HOWEVER CAUSED AND UNDER ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF WHISTLE COMMUNICATIONS IS ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 *
 * $FreeBSD: release/9.1.0/usr.sbin/ngctl/debug.c 160423 2006-07-17 08:35:47Z stefanf $
 */

#include <netgraph.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "ngctl.h"

static int DebugCmd(int ac, char **av);

const struct ngcmd debug_cmd = {
	DebugCmd,
	"debug [level]",
	"Get/set debugging verbosity level",
	"Without any argument, this command displays the current"
	" debugging verbosity level. If the argument is ``+'' or ``-''"
	" the debug level is incremented or decremented; otherwise,"
	" it must be an absolute numerical level.",
	{ NULL }
};

static int
DebugCmd(int ac, char **av)
{
	int level;

	/* Get arguments */
	switch (ac) {
	case 2:
		if (!strcmp(av[1], "+"))
			level = NgSetDebug(-1) + 1;
		else if (!strcmp(av[1], "-"))
			level = NgSetDebug(-1) - 1;
		else if ((level = atoi(av[1])) < 0)
			return (CMDRTN_USAGE);
		NgSetDebug(level);
		break;
	case 1:
		printf("Current debug level is %d\n", NgSetDebug(-1));
		break;
	default:
		return (CMDRTN_USAGE);
	}
	return (CMDRTN_OK);
}

